@@ -1,6 +1,7 @@
 from esphome import pins
 import esphome.config_validation as cv
 import esphome.codegen as cg
+from esphome.components.network import add_mdns_library
 from esphome.const import (
     CONF_DOMAIN,
     CONF_ID,
@@ -9,6 +10,7 @@ from esphome.const import (
     CONF_TYPE,
     CONF_USE_ADDRESS,
     ESP_PLATFORM_ESP32,
+    CONF_ENABLE_MDNS,
     CONF_GATEWAY,
     CONF_SUBNET,
     CONF_DNS1,
@@ -80,6 +82,7 @@ CONFIG_SCHEMA = cv.All(
             cv.Optional(CONF_PHY_ADDR, default=0): cv.int_range(min=0, max=31),
             cv.Optional(CONF_POWER_PIN): pins.gpio_output_pin_schema,
             cv.Optional(CONF_MANUAL_IP): MANUAL_IP_SCHEMA,
+            cv.Optional(CONF_ENABLE_MDNS, default=True): cv.boolean,
             cv.Optional(CONF_DOMAIN, default=".local"): cv.domain_name,
             cv.Optional(CONF_USE_ADDRESS): cv.string_strict,
             cv.Optional("hostname"): cv.invalid(
@@ -122,3 +125,6 @@ def to_code(config):
         cg.add(var.set_manual_ip(manual_ip(config[CONF_MANUAL_IP])))
 
     cg.add_define("USE_ETHERNET")
+
+    if config[CONF_ENABLE_MDNS]:
+        add_mdns_library()
