@@ -2,21 +2,29 @@ import hashlib
 
 from esphome import config_validation as cv, automation
 from esphome import codegen as cg
-from esphome.const import CONF_ID, CONF_INITIAL_VALUE, CONF_RESTORE_VALUE, CONF_TYPE, CONF_VALUE
+from esphome.const import (
+    CONF_ID,
+    CONF_INITIAL_VALUE,
+    CONF_RESTORE_VALUE,
+    CONF_TYPE,
+    CONF_VALUE,
+)
 from esphome.core import coroutine_with_priority
-from esphome.py_compat import IS_PY3
 
-globals_ns = cg.esphome_ns.namespace('globals')
-GlobalsComponent = globals_ns.class_('GlobalsComponent', cg.Component)
-GlobalVarSetAction = globals_ns.class_('GlobalVarSetAction', automation.Action)
+CODEOWNERS = ["@esphome/core"]
+globals_ns = cg.esphome_ns.namespace("globals")
+GlobalsComponent = globals_ns.class_("GlobalsComponent", cg.Component)
+GlobalVarSetAction = globals_ns.class_("GlobalVarSetAction", automation.Action)
 
 MULTI_CONF = True
-CONFIG_SCHEMA = cv.Schema({
-    cv.Required(CONF_ID): cv.declare_id(GlobalsComponent),
-    cv.Required(CONF_TYPE): cv.string_strict,
-    cv.Optional(CONF_INITIAL_VALUE): cv.string_strict,
-    cv.Optional(CONF_RESTORE_VALUE, default=False): cv.boolean,
-}).extend(cv.COMPONENT_SCHEMA)
+CONFIG_SCHEMA = cv.Schema(
+    {
+        cv.Required(CONF_ID): cv.declare_id(GlobalsComponent),
+        cv.Required(CONF_TYPE): cv.string_strict,
+        cv.Optional(CONF_INITIAL_VALUE): cv.string_strict,
+        cv.Optional(CONF_RESTORE_VALUE, default=False): cv.boolean,
+    }
+).extend(cv.COMPONENT_SCHEMA)
 
 
 # Run with low priority so that namespaces are registered first
@@ -31,26 +39,33 @@ def to_code(config):
         initial_value = cg.RawExpression(config[CONF_INITIAL_VALUE])
 
     rhs = GlobalsComponent.new(template_args, initial_value)
-    glob = cg.Pvariable(config[CONF_ID], rhs, type=res_type)
+    glob = cg.Pvariable(config[CONF_ID], rhs, res_type)
     yield cg.register_component(glob, config)
 
     if config[CONF_RESTORE_VALUE]:
         value = config[CONF_ID].id
-        if IS_PY3 and isinstance(value, str):
+        if isinstance(value, str):
             value = value.encode()
         hash_ = int(hashlib.md5(value).hexdigest()[:8], 16)
         cg.add(glob.set_restore_value(hash_))
 
 
-@automation.register_action('globals.set', GlobalVarSetAction, cv.Schema({
-    cv.Required(CONF_ID): cv.use_id(GlobalsComponent),
-    cv.Required(CONF_VALUE): cv.templatable(cv.string_strict),
-}))
+@automation.register_action(
+    "globals.set",
+    GlobalVarSetAction,
+    cv.Schema(
+        {
+            cv.Required(CONF_ID): cv.use_id(GlobalsComponent),
+            cv.Required(CONF_VALUE): cv.templatable(cv.string_strict),
+        }
+    ),
+)
 def globals_set_to_code(config, action_id, template_arg, args):
     full_id, paren = yield cg.get_variable_with_full_id(config[CONF_ID])
     template_arg = cg.TemplateArguments(full_id.type, *template_arg)
     var = cg.new_Pvariable(action_id, template_arg, paren)
-    templ = yield cg.templatable(config[CONF_VALUE], args, None,
-                                 to_exp=cg.RawExpression)
+    templ = yield cg.templatable(
+        config[CONF_VALUE], args, None, to_exp=cg.RawExpression
+    )
     cg.add(var.set_value(templ))
     yield var
