@@ -12,9 +12,20 @@ void HttpRequestComponent::dump_config() {
   ESP_LOGCONFIG(TAG, "  User-Agent: %s", this->useragent_);
 }
 
-void HttpRequestComponent::send() {
-  bool begin_status = false;
+void HttpRequestComponent::set_url(std::string url) {
+  this->url_ = url;
+  this->secure_ = url.compare(0, 6, "https:") == 0;
+
+  if (!this->last_url_.empty() && this->url_ != this->last_url_) {
+    // Close connection if url has been changed
+    this->client_.setReuse(false);
+    this->client_.end();
+  }
   this->client_.setReuse(true);
+}
+
+void HttpRequestComponent::send(const std::vector<HttpRequestResponseTrigger *> &response_triggers) {
+  bool begin_status = false;
   const String url = this->url_.c_str();
 #ifdef ARDUINO_ARCH_ESP32
   begin_status = this->client_.begin(url);
@@ -43,6 +54,9 @@ void HttpRequestComponent::send() {
   }
 
   int http_code = this->client_.sendRequest(this->method_, this->body_.c_str());
+  for (auto *trigger : response_triggers)
+    trigger->process(http_code);
+
   if (http_code < 0) {
     ESP_LOGW(TAG, "HTTP Request failed; URL: %s; Error: %s", this->url_.c_str(),
              HTTPClient::errorToString(http_code).c_str());
@@ -78,7 +92,10 @@ WiFiClient *HttpRequestComponent::get_wifi_client_() {
 }
 #endif
 
-void HttpRequestComponent::close() { this->client_.end(); }
+void HttpRequestComponent::close() {
+  this->last_url_ = this->url_;
+  this->client_.end();
+}
 
 const char *HttpRequestComponent::get_string() {
   static const String STR = this->client_.getString();
