@@ -56,8 +56,8 @@ void I2CComponent::raw_begin_transmission(uint8_t address) {
   ESP_LOGVV(TAG, "Beginning Transmission to 0x%02X:", address);
   this->wire_->beginTransmission(address);
 }
-bool I2CComponent::raw_end_transmission(uint8_t address) {
-  uint8_t status = this->wire_->endTransmission();
+bool I2CComponent::raw_end_transmission(uint8_t address, bool send_stop) {
+  uint8_t status = this->wire_->endTransmission(send_stop);
   ESP_LOGVV(TAG, "    Transmission ended. Status code: 0x%02X", status);
 
   switch (status) {
@@ -178,28 +178,67 @@ bool I2CComponent::write_byte_16(uint8_t address, uint8_t a_register, uint16_t d
 }
 
 void I2CDevice::set_i2c_address(uint8_t address) { this->address_ = address; }
+#ifdef USE_I2C_MULTIPLEXER
+void I2CDevice::set_i2c_multiplexer(I2CMultiplexer *multiplexer, uint8_t channel) {
+  ESP_LOGVV(TAG, "    Setting Multiplexer %p for channel %d", multiplexer, channel);
+  this->multiplexer_ = multiplexer;
+  this->channel_ = channel;
+}
+
+void I2CDevice::check_multiplexer_() {
+  if (this->multiplexer_ != nullptr) {
+    ESP_LOGVV(TAG, "Multiplexer setting channel to %d", this->channel_);
+    this->multiplexer_->set_channel(this->channel_);
+  }
+}
+#endif
+
 bool I2CDevice::read_bytes(uint8_t a_register, uint8_t *data, uint8_t len, uint32_t conversion) {  // NOLINT
+#ifdef USE_I2C_MULTIPLEXER
+  this->check_multiplexer_();
+#endif
   return this->parent_->read_bytes(this->address_, a_register, data, len, conversion);
 }
 bool I2CDevice::read_byte(uint8_t a_register, uint8_t *data, uint32_t conversion) {  // NOLINT
+#ifdef USE_I2C_MULTIPLEXER
+  this->check_multiplexer_();
+#endif
   return this->parent_->read_byte(this->address_, a_register, data, conversion);
 }
 bool I2CDevice::write_bytes(uint8_t a_register, const uint8_t *data, uint8_t len) {  // NOLINT
+#ifdef USE_I2C_MULTIPLEXER
+  this->check_multiplexer_();
+#endif
   return this->parent_->write_bytes(this->address_, a_register, data, len);
 }
 bool I2CDevice::write_byte(uint8_t a_register, uint8_t data) {  // NOLINT
+#ifdef USE_I2C_MULTIPLEXER
+  this->check_multiplexer_();
+#endif
   return this->parent_->write_byte(this->address_, a_register, data);
 }
 bool I2CDevice::read_bytes_16(uint8_t a_register, uint16_t *data, uint8_t len, uint32_t conversion) {  // NOLINT
+#ifdef USE_I2C_MULTIPLEXER
+  this->check_multiplexer_();
+#endif
   return this->parent_->read_bytes_16(this->address_, a_register, data, len, conversion);
 }
 bool I2CDevice::read_byte_16(uint8_t a_register, uint16_t *data, uint32_t conversion) {  // NOLINT
+#ifdef USE_I2C_MULTIPLEXER
+  this->check_multiplexer_();
+#endif
   return this->parent_->read_byte_16(this->address_, a_register, data, conversion);
 }
 bool I2CDevice::write_bytes_16(uint8_t a_register, const uint16_t *data, uint8_t len) {  // NOLINT
+#ifdef USE_I2C_MULTIPLEXER
+  this->check_multiplexer_();
+#endif
   return this->parent_->write_bytes_16(this->address_, a_register, data, len);
 }
 bool I2CDevice::write_byte_16(uint8_t a_register, uint16_t data) {  // NOLINT
+#ifdef USE_I2C_MULTIPLEXER
+  this->check_multiplexer_();
+#endif
   return this->parent_->write_byte_16(this->address_, a_register, data);
 }
 void I2CDevice::set_i2c_parent(I2CComponent *parent) { this->parent_ = parent; }
