@@ -1,8 +1,8 @@
 """Constants used by esphome."""
 
 MAJOR_VERSION = 1
-MINOR_VERSION = 17
-PATCH_VERSION = "2"
+MINOR_VERSION = 18
+PATCH_VERSION = "0"
 __short_version__ = f"{MAJOR_VERSION}.{MINOR_VERSION}"
 __version__ = f"{__short_version__}.{PATCH_VERSION}"
 
@@ -10,12 +10,14 @@ ESP_PLATFORM_ESP32 = "ESP32"
 ESP_PLATFORM_ESP8266 = "ESP8266"
 ESP_PLATFORMS = [ESP_PLATFORM_ESP32, ESP_PLATFORM_ESP8266]
 
-ALLOWED_NAME_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789_-"
+ALLOWED_NAME_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789-_"
+
 # Lookup table from ESP32 arduino framework version to latest platformio
 # package with that version
 # See also https://github.com/platformio/platform-espressif32/releases
 ARDUINO_VERSION_ESP32 = {
     "dev": "https://github.com/platformio/platform-espressif32.git",
+    "1.0.6": "platformio/espressif32@3.2.0",
     "1.0.5": "platformio/espressif32@3.1.1",
     "1.0.4": "platformio/espressif32@3.0.0",
     "1.0.3": "platformio/espressif32@1.10.0",
@@ -56,6 +58,7 @@ CONF_ACTION_ID = "action_id"
 CONF_ADDRESS = "address"
 CONF_ADDRESSABLE_LIGHT_ID = "addressable_light_id"
 CONF_ALPHA = "alpha"
+CONF_ALTITUDE = "altitude"
 CONF_AND = "and"
 CONF_AP = "ap"
 CONF_ARDUINO_VERSION = "arduino_version"
@@ -94,6 +97,7 @@ CONF_BUSY_PIN = "busy_pin"
 CONF_CALIBRATE_LINEAR = "calibrate_linear"
 CONF_CALIBRATION = "calibration"
 CONF_CAPACITANCE = "capacitance"
+CONF_CAPACITY = "capacity"
 CONF_CARRIER_DUTY_PERCENT = "carrier_duty_percent"
 CONF_CARRIER_FREQUENCY = "carrier_frequency"
 CONF_CERTIFICATE = "certificate"
@@ -112,6 +116,7 @@ CONF_CO2 = "co2"
 CONF_CODE = "code"
 CONF_COLD_WHITE = "cold_white"
 CONF_COLD_WHITE_COLOR_TEMPERATURE = "cold_white_color_temperature"
+CONF_COLOR = "color"
 CONF_COLOR_CORRECT = "color_correct"
 CONF_COLOR_TEMPERATURE = "color_temperature"
 CONF_COLORS = "colors"
@@ -127,7 +132,9 @@ CONF_CONDUCTIVITY = "conductivity"
 CONF_CONTRAST = "contrast"
 CONF_COOL_ACTION = "cool_action"
 CONF_COOL_MODE = "cool_mode"
+CONF_COUNT = "count"
 CONF_COUNT_MODE = "count_mode"
+CONF_COURSE = "course"
 CONF_CRON = "cron"
 CONF_CS_PIN = "cs_pin"
 CONF_CSS_INCLUDE = "css_include"
@@ -176,14 +183,17 @@ CONF_ECHO_PIN = "echo_pin"
 CONF_EFFECT = "effect"
 CONF_EFFECTS = "effects"
 CONF_ELSE = "else"
+CONF_ENABLE_MDNS = "enable_mdns"
 CONF_ENABLE_PIN = "enable_pin"
 CONF_ENABLE_TIME = "enable_time"
 CONF_ENERGY = "energy"
 CONF_ENTITY_ID = "entity_id"
 CONF_ESP8266_RESTORE_FROM_FLASH = "esp8266_restore_from_flash"
 CONF_ESPHOME = "esphome"
+CONF_ETHERNET = "ethernet"
 CONF_EVENT = "event"
 CONF_EXPIRE_AFTER = "expire_after"
+CONF_EXTERNAL_COMPONENTS = "external_components"
 CONF_EXTERNAL_VCC = "external_vcc"
 CONF_FALLING_EDGE = "falling_edge"
 CONF_FAMILY = "family"
@@ -204,11 +214,14 @@ CONF_FILE = "file"
 CONF_FILTER = "filter"
 CONF_FILTER_OUT = "filter_out"
 CONF_FILTERS = "filters"
+CONF_FINGER_ID = "finger_id"
+CONF_FINGERPRINT_COUNT = "fingerprint_count"
 CONF_FLASH_LENGTH = "flash_length"
 CONF_FOR = "for"
 CONF_FORCE_UPDATE = "force_update"
 CONF_FORMALDEHYDE = "formaldehyde"
 CONF_FORMAT = "format"
+CONF_FORWARD_ACTIVE_ENERGY = "forward_active_energy"
 CONF_FREQUENCY = "frequency"
 CONF_FROM = "from"
 CONF_FULL_UPDATE_EVERY = "full_update_every"
@@ -270,6 +283,9 @@ CONF_KEEP_ON_TIME = "keep_on_time"
 CONF_KEEPALIVE = "keepalive"
 CONF_KEY = "key"
 CONF_LAMBDA = "lambda"
+CONF_LAST_CONFIDENCE = "last_confidence"
+CONF_LAST_FINGER_ID = "last_finger_id"
+CONF_LATITUDE = "latitude"
 CONF_LENGTH = "length"
 CONF_LEVEL = "level"
 CONF_LG = "lg"
@@ -282,6 +298,7 @@ CONF_LOCAL = "local"
 CONF_LOG_TOPIC = "log_topic"
 CONF_LOGGER = "logger"
 CONF_LOGS = "logs"
+CONF_LONGITUDE = "longitude"
 CONF_LOW = "low"
 CONF_LOW_VOLTAGE_REFERENCE = "low_voltage_reference"
 CONF_MAC_ADDRESS = "mac_address"
@@ -328,11 +345,13 @@ CONF_NAME = "name"
 CONF_NBITS = "nbits"
 CONF_NEC = "nec"
 CONF_NETWORKS = "networks"
+CONF_NEW_PASSWORD = "new_password"
 CONF_NOISE_LEVEL = "noise_level"
 CONF_NUM_ATTEMPTS = "num_attempts"
 CONF_NUM_CHANNELS = "num_channels"
 CONF_NUM_CHIPS = "num_chips"
 CONF_NUM_LEDS = "num_leds"
+CONF_NUM_SCANS = "num_scans"
 CONF_NUMBER = "number"
 CONF_OFF_MODE = "off_mode"
 CONF_OFFSET = "offset"
@@ -342,7 +361,14 @@ CONF_ON_BLE_MANUFACTURER_DATA_ADVERTISE = "on_ble_manufacturer_data_advertise"
 CONF_ON_BLE_SERVICE_DATA_ADVERTISE = "on_ble_service_data_advertise"
 CONF_ON_BOOT = "on_boot"
 CONF_ON_CLICK = "on_click"
+CONF_ON_CONNECT = "on_connect"
+CONF_ON_DISCONNECT = "on_disconnect"
 CONF_ON_DOUBLE_CLICK = "on_double_click"
+CONF_ON_ENROLLMENT_DONE = "on_enrollment_done"
+CONF_ON_ENROLLMENT_FAILED = "on_enrollment_failed"
+CONF_ON_ENROLLMENT_SCAN = "on_enrollment_scan"
+CONF_ON_FINGER_SCAN_MATCHED = "on_finger_scan_matched"
+CONF_ON_FINGER_SCAN_UNMATCHED = "on_finger_scan_unmatched"
 CONF_ON_JSON_MESSAGE = "on_json_message"
 CONF_ON_LOOP = "on_loop"
 CONF_ON_MESSAGE = "on_message"
@@ -380,6 +406,7 @@ CONF_PAGE_ID = "page_id"
 CONF_PAGES = "pages"
 CONF_PANASONIC = "panasonic"
 CONF_PASSWORD = "password"
+CONF_PATH = "path"
 CONF_PAYLOAD = "payload"
 CONF_PAYLOAD_AVAILABLE = "payload_available"
 CONF_PAYLOAD_NOT_AVAILABLE = "payload_not_available"
@@ -441,6 +468,7 @@ CONF_RESTORE_MODE = "restore_mode"
 CONF_RESTORE_STATE = "restore_state"
 CONF_RESTORE_VALUE = "restore_value"
 CONF_RETAIN = "retain"
+CONF_REVERSE_ACTIVE_ENERGY = "reverse_active_energy"
 CONF_RGB_ORDER = "rgb_order"
 CONF_RGBW = "rgbw"
 CONF_RISING_EDGE = "rising_edge"
@@ -456,6 +484,7 @@ CONF_RX_ONLY = "rx_only"
 CONF_RX_PIN = "rx_pin"
 CONF_SAFE_MODE = "safe_mode"
 CONF_SAMSUNG = "samsung"
+CONF_SATELLITES = "satellites"
 CONF_SCAN = "scan"
 CONF_SCL = "scl"
 CONF_SCL_PIN = "scl_pin"
@@ -463,10 +492,12 @@ CONF_SDA = "sda"
 CONF_SDO_PIN = "sdo_pin"
 CONF_SECOND = "second"
 CONF_SECONDS = "seconds"
+CONF_SECURITY_LEVEL = "security_level"
 CONF_SEGMENTS = "segments"
 CONF_SEL_PIN = "sel_pin"
 CONF_SEND_EVERY = "send_every"
 CONF_SEND_FIRST_AT = "send_first_at"
+CONF_SENSING_PIN = "sensing_pin"
 CONF_SENSOR = "sensor"
 CONF_SENSOR_ID = "sensor_id"
 CONF_SENSORS = "sensors"
@@ -485,6 +516,7 @@ CONF_SLEEP_DURATION = "sleep_duration"
 CONF_SLEEP_PIN = "sleep_pin"
 CONF_SLEEP_WHEN_DONE = "sleep_when_done"
 CONF_SONY = "sony"
+CONF_SOURCE = "source"
 CONF_SPEED = "speed"
 CONF_SPEED_COMMAND_TOPIC = "speed_command_topic"
 CONF_SPEED_COUNT = "speed_count"
@@ -496,6 +528,7 @@ CONF_SSL_FINGERPRINTS = "ssl_fingerprints"
 CONF_STATE = "state"
 CONF_STATE_TOPIC = "state_topic"
 CONF_STATIC_IP = "static_ip"
+CONF_STATUS = "status"
 CONF_STEP_MODE = "step_mode"
 CONF_STEP_PIN = "step_pin"
 CONF_STOP = "stop"
@@ -590,8 +623,11 @@ ICON_ACCELERATION = "mdi:axis-arrow"
 ICON_ACCELERATION_X = "mdi:axis-x-arrow"
 ICON_ACCELERATION_Y = "mdi:axis-y-arrow"
 ICON_ACCELERATION_Z = "mdi:axis-z-arrow"
+ICON_ACCOUNT = "mdi:account"
+ICON_ACCOUNT_CHECK = "mdi:account-check"
 ICON_ARROW_EXPAND_VERTICAL = "mdi:arrow-expand-vertical"
 ICON_BATTERY = "mdi:battery"
+ICON_BLUETOOTH = "mdi:bluetooth"
 ICON_BRIEFCASE_DOWNLOAD = "mdi:briefcase-download"
 ICON_BRIGHTNESS_5 = "mdi:brightness-5"
 ICON_BUG = "mdi:bug"
@@ -599,7 +635,9 @@ ICON_CHECK_CIRCLE_OUTLINE = "mdi:check-circle-outline"
 ICON_CHEMICAL_WEAPON = "mdi:chemical-weapon"
 ICON_COUNTER = "mdi:counter"
 ICON_CURRENT_AC = "mdi:current-ac"
+ICON_DATABASE = "mdi:database"
 ICON_EMPTY = ""
+ICON_FINGERPRINT = "mdi:fingerprint"
 ICON_FLASH = "mdi:flash"
 ICON_FLASK = "mdi:flask"
 ICON_FLASK_OUTLINE = "mdi:flask-outline"
@@ -607,6 +645,7 @@ ICON_FLOWER = "mdi:flower"
 ICON_GAS_CYLINDER = "mdi:gas-cylinder"
 ICON_GAUGE = "mdi:gauge"
 ICON_GRAIN = "mdi:grain"
+ICON_KEY_PLUS = "mdi:key-plus"
 ICON_LIGHTBULB = "mdi:lightbulb"
 ICON_MAGNET = "mdi:magnet"
 ICON_MOLECULE_CO2 = "mdi:molecule-co2"
@@ -623,6 +662,7 @@ ICON_RULER = "mdi:ruler"
 ICON_SCALE = "mdi:scale"
 ICON_SCALE_BATHROOM = "mdi:scale-bathroom"
 ICON_SCREEN_ROTATION = "mdi:screen-rotation"
+ICON_SECURITY = "mdi:security"
 ICON_SIGN_DIRECTION = "mdi:sign-direction"
 ICON_SIGNAL = "mdi:signal-distance-variant"
 ICON_SIGNAL_DISTANCE_VARIANT = "mdi:signal"
