@@ -16,12 +16,14 @@
 #include "esphome/components/ethernet/ethernet_component.h"
 #endif
 
+#ifdef USE_MDNS
 #ifdef ARDUINO_ARCH_ESP32
 #include <ESPmDNS.h>
 #endif
 #ifdef ARDUINO_ARCH_ESP8266
 #include <ESP8266mDNS.h>
 #endif
+#endif
 
 namespace esphome {
 
@@ -39,10 +41,15 @@ bool network_is_connected() {
   return false;
 }
 
-#ifdef ARDUINO_ARCH_ESP8266
+#if defined(ARDUINO_ARCH_ESP8266) && defined(USE_MDNS)
 bool mdns_setup;
 #endif
 
+#ifndef WEBSERVER_PORT
+static const uint8_t WEBSERVER_PORT = 80;
+#endif
+
+#ifdef USE_MDNS
 #ifdef ARDUINO_ARCH_ESP8266
 void network_setup_mdns(IPAddress address, int interface) {
   // Latest arduino framework breaks mDNS for AP interface
@@ -65,16 +72,21 @@ void network_setup_mdns(IPAddress address, int interface) {
       MDNS.addServiceTxt("esphomelib", "tcp", "mac", get_mac_address().c_str());
     } else {
 #endif
-      // Publish "http" service if not using native API.
+      // Publish "http" service if not using native API nor the webserver component
       // This is just to have *some* mDNS service so that .local resolution works
-      MDNS.addService("http", "tcp", 80);
+      MDNS.addService("http", "tcp", WEBSERVER_PORT);
       MDNS.addServiceTxt("http", "tcp", "version", ESPHOME_VERSION);
 #ifdef USE_API
     }
+#endif
+#ifdef USE_PROMETHEUS
+    MDNS.addService("prometheus-http", "tcp", WEBSERVER_PORT);
 #endif
   }
+#endif
+
   void network_tick_mdns() {
-#ifdef ARDUINO_ARCH_ESP8266
+#if defined(ARDUINO_ARCH_ESP8266) && defined(USE_MDNS)
     if (mdns_setup)
       MDNS.update();
 #endif
